# /var/www/abstractendeavors/secure-files/big_man/flask_app/login_app/functions/query_utils.py

from abstract_database import connectionManager
import psycopg
from psycopg.rows import dict_row
from abstract_utilities import initialize_call_log

# Initialize connectionManager once (using your .env path if needed)



def get_cur_conn(use_dict_cursor=True):
    """
    Get a database connection and a RealDictCursor.
    Returns:
        tuple: (cursor, connection)
    """
    conn = connectionManager().get_db_connection()
    cur = conn.cursor(row_factory=dict_row) if use_dict_cursor else conn.cursor()
    return cur, conn


# -----------------------------
# SELECT helpers
# -----------------------------
def select_all(query: str, *args):
    """Run SELECT returning all rows."""
    cur, conn = get_cur_conn()
    try:
        cur.execute(query, args) if args else cur.execute(query)
        return cur.fetchall()
    finally:
        cur.close()
        conn.close()


def select_distinct_rows(query: str, *args):
    """Run SELECT returning all rows as list[dict]."""
    return select_all(query, *args)


def select_rows(query: str, *args):
    """Run SELECT returning one row (or None)."""
    cur, conn = get_cur_conn()
    try:
        cur.execute(query, args) if args else cur.execute(query)
        row = cur.fetchone()
        return row if row else []
    finally:
        cur.close()
        conn.close()


# -----------------------------
# Write helpers (INSERT/UPDATE/DELETE)
# -----------------------------
def insert_query(query: str, *args):
    """
    Run INSERT. If RETURNING clause exists, return the id (or None).
    """
    cur, conn = get_cur_conn()
    try:
        cur.execute(query, args)
        new_id = None
        try:
            res = cur.fetchone()
            if res and "id" in res:
                new_id = res["id"]
        except psycopg.ProgrammingError:
            pass
        conn.commit()
        return new_id
    finally:
        cur.close()
        conn.close()


def execute_query(query: str, *args):
    """
    Run UPDATE/DELETE/INSERT (no fetch).
    Returns number of rows affected.
    """
    cur, conn = get_cur_conn()
    try:
        cur.execute(query, args)
        affected = cur.rowcount
        conn.commit()
        return affected
    finally:
        cur.close()
        conn.close()



from .imports import *
from .endpoints import (
    secure_env_bp,
    secure_login_bp,
    secure_files_bp,
    secure_users_bp,
    secure_views_bp,
    secure_upload_bp,
    secure_remove_bp,
    secure_logout_bp,
    secure_register_bp,
    secure_download_bp,
    secure_settings_bp,
    change_passwords_bp,
    secure_endpoints_bp,
    )
from .templates import *
#from .chats import secure_chat_bp
bp_list = [
    secure_env_bp,
    secure_login_bp,
    secure_files_bp,
    secure_users_bp,
    secure_views_bp,
    secure_upload_bp,
    secure_remove_bp,
    secure_logout_bp,
    secure_register_bp,
    secure_download_bp,
    secure_settings_bp,
    change_passwords_bp,
    secure_endpoints_bp,
    ]



import os, datetime, jwt
from functools import wraps
from abstract_flask import request, jsonify
from .token_utils import decode_token
from ..user_store.table_utils.routes import is_token_blacklisted, ensure_blacklist_table_exists
from ..user_store.get_users import get_user_by_username
ensure_blacklist_table_exists()

def login_required(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        auth_header = request.headers.get("Authorization", None)
        if not auth_header:
            return jsonify({"message": "Missing Authorization header"}), 401

        parts = auth_header.split()
        if parts[0].lower() != "bearer" or len(parts) != 2:
            return jsonify({"message": "Invalid Authorization header format"}), 401

        token = parts[1]

        # 1) Check if token is blacklisted
        #if is_token_blacklisted(token):
        #    return jsonify({"message": "Token has been revoked"}), 401

        # 2) Decode and validate the token
        try:
            payload = decode_token(token)
        except jwt.ExpiredSignatureError:
            return jsonify({"message": "Token expired"}), 401
        except jwt.InvalidTokenError:
            return jsonify({"message": "Invalid token"}), 401

        # 3) Extract username and admin flag
        username = payload.get("username")
        is_admin = payload.get("is_admin", False)

        # 4) Determine integer user_id
        if "sub" in payload:
            # If you issued the JWT with the user’s numeric ID in `sub`
            user_id = int(payload["sub"])
        else:
            # Otherwise look it up from your user store
            user = get_user_by_username(username)
            if not user:
                return jsonify({"message": "Unknown user"}), 404
            user_id = user["id"]

        # 5) Attach to request for downstream handlers
        request.user = {
            "id":       user_id,
            "username": username,
            "is_admin": is_admin
        }

        # 6) Call the wrapped view
        return f(*args, **kwargs)
    return decorated



from .base_dir import *
DOCUMENTS_MEDIA_DIR = get_media_dir("documents")
DOWNLOADS_MEDIA_DIR = get_media_dir("downloads")
IMAGES_MEDIA_DIR = get_media_dir("images")
ORBITSIMULATOR_MEDIA_DIR = get_media_dir("orbitSimulator")
PDFS_MEDIA_DIR = get_media_dir("pdfs")
THEDAILYDIALECTICS_MEDIA_DIR = get_media_dir("thedailydialectics")
UPLOADS_MEDIA_DIR = get_media_dir("uploads")
USERS_MEDIA_DIR = get_media_dir("users")
VIDEOS_MEDIA_DIR = get_media_dir("videos")
UPLOAD_FOLDER = ABS_UPLOAD_DIR = UPLOADS_MEDIA_DIR
ABS_UPLOAD_ROOT = ABS_UPLOAD_DIR = UPLOADS_MEDIA_DIR
##



from ..imports import *

secure_upload_bp, logger = get_bp(
    "secure_upload_bp",
    __name__
)
def clean_rel_path(path: str) -> str:
    # prevent Flask placeholders or traversal junk
    if "<" in path or ">" in path:
        return os.path.basename(path)
    return secure_filename(path)
def get_upload_creds(req) -> tuple[str, "FileStorage"]:
    """
    Extracts and validates user_name and file from the request.
    Aborts with 400 if missing.
    """
    user_name = req.user.get("username")
    if not user_name:
        abort(400, "Missing username")

    file = req.files.get("file")
    if not file or not file.filename:
        abort(400, description="No file provided")

    return user_name, file
@secure_upload_bp.route("/api/secure-files/upload", methods=["POST", "GET"], strict_slashes=False)
@login_required
def upload_file(rel_path: str | None = None):
    initialize_call_log()

    # 1) get and validate creds
    user_name, file = get_upload_creds(request)
    logger.info(f"user_name={user_name}")
    logger.info(f"file={file}")

    # 2) determine where to save
    is_public = user_name == "public"
    user_dir = get_user_upload_dir(request, user_name, user_upload_dir=is_public)

    route_rel = rel_path
    form_rel = request.form.get("rel_path")
    basename = secure_filename(file.filename or "file")

    rel = route_rel or form_rel or basename
    rel = clean_rel_path(rel)

    full_path = safe_join(user_dir, rel)
    file.save(full_path)  # ✅ save once only

    db_path = os.path.relpath(full_path, user_dir)


    file_id = insert_any_combo(
        table_name="uploads",
        insert_map={
            "filename": secure_filename(basename),
            "filepath": db_path,
            "uploader_id": user_name,
            "shareable": request.form.get("shareable", False),
            "download_count": request.form.get("download_count", 0),
            "download_limit": request.form.get("download_limit"),
            "share_password": request.form.get("share_password"),
        },
        returning="id",
    )

    # 5) return a consistent response
    return jsonify({
        "message":     "File uploaded successfully.",
        "filename":    secure_filename(basename),
        "filepath":    db_path,
        "file_id":     file_id,
        "uploader_id": user_name,
    }), 200



import logging
from abstract_flask import get_Flask_app
from .app import bp_list

def login_app(
    allowed_origins=None,
    name=None
    ):
    ALLOWED_ORIGINS = allowed_origins or ["*"]
    name= name or "abstract_logins"
    return get_Flask_app(
        name=name,
        bp_list=bp_list,
        allowed_origins=allowed_origins
        )



from ..imports import *
secure_endpoints_bp,logger = get_bp(
    "secure_endpoints",
    __name__,
    url_prefix=SECURE_FILES_PREFIX
      # <-- everything in this Blueprint sits under /secure-files
)
@secure_endpoints_bp.route("/api/secure-files/secure_endpoints", methods=["GET","POST"])
@secure_endpoints_bp.route("/api/secure-files/secure_endpoints/", methods=["GET","POST"])
def getEndpoints():
    import sys, os, importlib
    try:
        from ....__init__ import login_app
        app = login_app()
        endpoints=[]
        for rule in app.url_map.iter_rules():
            
            # skip dynamic parameters if desired, include all
            methods = sorted(rule.methods - {"HEAD", "OPTIONS"})
            endpoints.append((rule.rule, ", ".join(methods)))
        rules = sorted(endpoints, key=lambda x: x[0])
        return jsonify(rules), 200
    finally:
        sys.path.pop(0)



import getpass,bcrypt
from abstract_utilities import get_logFile,initialize_call_log
logging = get_logFile('pass_utils')
def generate_salt(rounds=None):
    rounds = rounds or 10
    salt = bcrypt.gensalt(rounds=rounds)
    return salt

def verify_password(plaintext_pwd: str, stored_hash: str) -> bool:
    """
    Returns True if plaintext matches the bcrypt stored_hash, else False.
    If the stored_hash isn’t a valid bcrypt hash, logs and returns False.
    """
    if not plaintext_pwd or not stored_hash:
        return False
    try:
        return bcrypt.checkpw(
            plaintext_pwd.encode("utf8"),
            stored_hash.encode("utf8")
        )
    except ValueError as e:
        # invalid salt or hash format
        logging.error("Invalid bcrypt hash in DB: %s", e)
        return False

def input_plain_text():
    initialize_call_log()
    plaintext = getpass.getpass("Enter new admin password: ").strip()
    if not plaintext:
        print("✘ Password cannot be empty. Exiting.")
        exit(1)
    return plaintext


def bcrypt_plain_text(plaintext_pwd,rounds=None):
    salt = generate_salt(rounds=rounds)
    plaintext_pwd = plaintext_pwd.encode("utf-8")
    encrypted_bcrypt_hash = bcrypt.hashpw(plaintext_pwd,salt)
    bcrypt_hash = encrypted_bcrypt_hash.decode("utf-8")
    return bcrypt_hash



# /flask_app/login_app/endpoints/files/secure_files.py
from ..imports import *
# Initialize Blueprint
secure_remove_bp,logger = get_bp(
    name="secure_remove",
    abs_path=__name__

)
@secure_remove_bp.route("/api/secure-files/remove", methods=["POST", "GET"])
@login_required
def remove_file():
    """
    Endpoint to remove a file securely.

    Expects JSON data with 'id', 'file_id', 'filepath', or 'rel_path'.
    Returns JSON response with status and message.
    """
    args, datas, username = get_args_jwargs_user_req(request)
    logger.info(f"username==={username}")
    results = []
    for data in make_list(datas):
        err = f"Remove file request: {data}"
        try:
            msg, err_code = secure_remove(data, username)
            logger.info()
        except Exception as e:
            err = f"Server error in remove_file: {e}"
            err_code =404
        #logger.error(err)
        #call_response = get_json_call_response(msg, err_code)
        #results.append(call_response)
    return get_json_call_response(True, 200)
@secure_remove_bp.route("/api/secure-files/remove_files", methods=["POST", "GET"])
@login_required
def removFile():
    """
    Endpoint to remove a file securely.

    Expects JSON data with 'id', 'file_id', 'filepath', or 'rel_path'.
    Returns JSON response with status and message.
    """
    result = extract_request_data(request)
    username = result.get('user')
    args = result.get('args', [])
    data = result.get('json',{})
    results = []
    for data in make_list(data):
        msg, err_code = secure_remove(data=data, username=username,req=request)
        logger.info(f"Remove file msg: {msg}")
        logger.info(f"Remove file err_code: {err_code}")
        call_response = get_json_call_response(msg, err_code)
        results.append(call_response)
    return get_json_call_response(True, 200)



import os
def get_joined(*paths):
    joined_path = os.path.join(*paths)
    return joined_path
def make_joined(*paths):
    joined_path = get_joined(*paths)  # assumes returns a string path

    if not os.path.exists(joined_path):
        root, ext = os.path.splitext(joined_path)

        # Heuristic: if it ends with a separator, treat as dir regardless of ext
        if joined_path.endswith(os.sep):
            os.makedirs(joined_path, exist_ok=True)
        # If there is an extension, assume it's a file path: create parent dir
        elif ext:
            os.makedirs(os.path.dirname(joined_path) or ".", exist_ok=True)
        # Otherwise, treat as a directory path: create it directly
        else:
            os.makedirs(joined_path, exist_ok=True)

    return joined_path
ABS_BASE_DIR=get_home_dir()
MEDIA_DIR=os.path.join(ABS_BASE_DIR,'media')
def get_base_dir(path):
    return get_joined(ABS_BASE_DIR, path)

def get_media_dir(path):
    return make_joined(MEDIA_DIR,path)
FUNCTIONS_DIR = get_base_dir('functions')
MODULES_DIR = get_base_dir('modules')
HTML_DIR = get_base_dir('html')
API_DIR = get_base_dir('api')
def get_functions_dir(path):
    return make_joined(FUNCTIONS_DIR,path)
def get_modules_dir(path):
    return make_joined(MODULES_DIR,path)
def get_html_dir(path):
    return make_joined(HTML_DIR,path)
def get_api_dir(path):
    return make_joined(API_DIR,path)






# /flask_app/login_app/endpoints/files/secure_files.py
from ..imports import *
# Correct get_bp signature:  get_bp(name, *, url_prefix=None, static_folder=None)
secure_files_bp, logger = get_bp(
    "secure_files_bp",
    __name__


)
@secure_files_bp.route("/api/secure-files/list", methods=["GET", "POST"], strict_slashes=False)
@login_required
def list_files():
    user_name = get_user_name(req=request)
    items = get_upload_items(
        req=request,
        user_name=user_name,
        include_untracked=False,   # ← skip the FS-scan on your "initial" list call
        prune_files=True
    )
    return get_json_call_response(items, 200)

@secure_files_bp.route("/api/secure-files/userInfo", methods=["GET", "POST"], strict_slashes=False)
@login_required
def list_user_info():
    user_name = get_user_name(req=request)
    user_dir = get_user_upload_dir(request, user_name)
    items = get_upload_items(
        req=request,
        user_name=user_name,
        include_untracked=False   # ← skip the FS-scan on your "initial" list call
    )
    data = {
        "user_name": user_name,
        "user_dir": user_dir,
        "list":items
        
    }
    return get_json_call_response(data, 200)






from .imports import *
from .auth_utils import *
def get_req_data(req=None,data=None):
   if data:
      return data
   result = extract_request_data(req) or {}
   data = result.get('json',{})
   return data
def get_uploader_id(req=None,data=None):
   data = get_req_data(req=req,data=data)
   uploader_id = data.get('uploader_id')
   return uploader_id

def is_user_uploader(req=None,data=None,user_id=None):
   if not user_id:
      user_id,user_err = get_user_id_from_request(req)
   uploader_id = get_uploader_id(req=req,data=data)
   return user_id and uploader_id and user_id == uploader_id
def get_user_upload_dir(req=None, user_name=None, user_upload_dir=None):
    user_name = get_user_name(req=req, user_name=user_name)
    return os.path.join(USERS_DIR, user_name)

def get_user_from_path(path):
    if path.startswith(USERS_DIR):
        return path.split(USERS_DIR)[1].split('/')[0]
    return filepath.split('/')[0]
def get_glob_files(req=None,user_name=None,user_upload_dir=None):
    user_upload_dir = get_user_upload_dir(req=req,user_name=user_name,user_upload_dir=user_upload_dir)
    pattern = os.path.join(user_upload_dir, "**/*")  # include all files recursively
    glob_files = glob.glob(pattern, recursive=True)
    logger.info(f"glob_files == {glob_files}")
    return glob_files
def get_file_id(file_dict=None, row=None):
    """
    Derive the file ID from a file dictionary or row data.
    
    Args:
        file_dict (dict, optional): Dictionary containing file data (e.g., {'id': 123, 'filename': 'example.txt'}).
        row (dict, optional): Dictionary containing row data (e.g., {'fileId': '123'}).
    
    Returns:
        int: The numeric file ID.
    
    Raises:
        ValueError: If file ID cannot be derived from file_dict or row.
    """
    if file_dict and 'id' in file_dict and file_dict['id'] is not None:
        return int(file_dict['id'])  # Numeric ID from file dictionary
    if row and 'fileId' in row and row['fileId'] is not None:
        return int(row['fileId'])  # Numeric ID from row dictionary
    raise ValueError('Unable to derive fileId: no file.id or row.fileId')
def get_full_path(file_path: str, user: object = None,row: dict = None,req: object = None,data: object = None,user_name: object = None) -> Path:
    """
    Resolve a user's file path under /var/www/media/users/<uploader_id> safely.
    - file_path: user-supplied relative path (e.g., 'project/a.txt' or '../evil')
    - row: dict that may contain 'uploader_id'
    - user: object/str that may indicate the id via .id or itself
    """
    row = row or {}
    uploader_id = user or user_name or row.get("uploader_id") or get_uploader_id(req=req,data=data)
    if uploader_id is None:
       if user is not None:
           uploader_id = getattr(user, "id", user)
    if uploader_id is None:
       raise ValueError("uploader_id is required (provide via row['uploader_id'] or user)")

    user_dir = safe_join(USERS_DIR,str(uploader_id))
    # Ensure the user directory reference is real; create if you want:
    user_dir.mkdir(parents=True, exist_ok=True)

    # Return a safe, normalized absolute Path under the user_dir
    return safe_join(user_dir, file_path)
##def get_full_path(filepath: str, user_name: str) -> str:
##    user_dir = get_user_upload_dir(user_name=user_name)
##    return safe_join(user_dir, filepath)  # safe_join ensures no escape
##




from ...query_utils import insert_query, select_rows,initialize_call_log
def ensure_blacklist_table_exists():
    """
    Create the 'blacklisted_tokens' table if it doesn’t already exist.
    Columns:
      - token (TEXT): the JWT string
      - blacklisted_on (TIMESTAMPTZ): timestamp when it was added
    """
    initialize_call_log()
    query = """
    CREATE TABLE IF NOT EXISTS blacklisted_tokens (
      token           TEXT PRIMARY KEY,
      blacklisted_on  TIMESTAMPTZ DEFAULT NOW()
    );
    """
    insert_query(query)

def is_token_blacklisted(token: str) -> bool:
    """
    Returns True if the given token is already in blacklisted_tokens.
    """
    query = "SELECT 1 FROM blacklisted_tokens WHERE token = %s;"
    row = select_rows(query, token)
    return row is not None

def blacklist_token(token: str) -> None:
    """
    Insert the given JWT into blacklisted_tokens.
    """
    initialize_call_log()
    query = "INSERT INTO blacklisted_tokens (token) VALUES (%s) ON CONFLICT DO NOTHING;"
    insert_query(query, token)



from ...query_utils import insert_query, select_rows,initialize_call_log
def create_users_table():
    """
    Create the 'users' table (if not exists) and an updated_at trigger.
    Uses connectionManager via query_utils.
    """
    initialize_call_log()
    # 1) Create the users table if it does not exist
    query = """
    CREATE TABLE IF NOT EXISTS users (
      username      VARCHAR(255) PRIMARY KEY,
      password_hash TEXT           NOT NULL,
      is_admin      BOOLEAN        NOT NULL DEFAULT FALSE,
      created_at    TIMESTAMPTZ    DEFAULT NOW(),
      updated_at    TIMESTAMPTZ    DEFAULT NOW()
    );
    """
    
    insert_query(query)
def ensure_trigger_exists():
    initialize_call_log()
    # 2) Create (or replace) the trigger function to update updated_at
    query = """
    CREATE OR REPLACE FUNCTION update_users_updated_at()
    RETURNS TRIGGER AS $$
    BEGIN
      NEW.updated_at := NOW();
      RETURN NEW;
    END;
    $$ LANGUAGE plpgsql;
    """
    insert_query(query)
def create_trigger():
    initialize_call_log()
    # 3) Create the trigger if it doesn’t already exist
    query = """
    DO $$
    BEGIN
      IF NOT EXISTS (
        SELECT 1 FROM pg_trigger WHERE tgname = 'users_updated_at_trigger'
      ) THEN
        CREATE TRIGGER users_updated_at_trigger
        BEFORE UPDATE ON users
        FOR EACH ROW
        EXECUTE PROCEDURE update_users_updated_at();
      END IF;
    END;
    $$;
    """
    insert_query(query)
def ensure_users_table_exists():
    create_users_table()
    ensure_trigger_exists()
    create_trigger()



from ..imports import *
from flask import Response

def send_via_nginx(abs_path: str, filename: str):
    """
    Return a response telling Nginx to serve the file directly.
    """
    rel_path = str(abs_path).split("/24T/media/")[-1]  # adjust to your base
    resp = Response()
    resp.headers["Content-Type"] = "application/octet-stream"
    resp.headers["Content-Disposition"] = f'attachment; filename="{filename}"'
    resp.headers["X-Accel-Redirect"] = f"/protected/{rel_path}"
    return resp
secure_download_bp, logger = get_bp(
    "secure_download",
    __name__
)

@secure_download_bp.route("/api/secure-files/download", methods=['GET',"POST"])
@login_required
def downloadFile():
    #initialize_call_log()
    
    args,datas,username = get_args_jwargs_user_req(request)
    logger.info(datas)
            # Validate and collect files
    valid_files = []
    errors = []
    for i,data in enumerate(make_list(datas)):
        data = dict(data)
        abs_path=None
        filename = None
        err = False
        filepath = data.get('filepath')
        if filepath:
          abs_path,filename = get_path_and_filename(filepath)
        filename = data.get('filename')
        
        logger.info(f"dict dsata =={data}")
        if not filename or not abs_path:
           abs_path, filename, err = get_download(
              data=data,
              username=username
              )
        logger.info(f"abs_path =={filepath}")
        logger.info(f"err =={err}")
        logger.info(f"filename =={filename}")
        
        if err:
            errors.append((filename, err))
            continue
        if isinstance(filename, int):
            errors.append((filename, 'Invalid filename'))
            continue
        valid_files.append((abs_path, filename))

    if len(valid_files) >1:
        # safety-check – abs_path/filename should never be None here
        if not abs_path or not filename:
            return get_json_call_response({"error": "NO_FILE_FOUND"}, 404)
                # Create ZIP archive in memory
            zip_buffer = io.BytesIO()
            with zipfile.ZipFile(zip_buffer, 'w', zipfile.ZIP_DEFLATED) as zip_file:
                for abs_path, filename in valid_files:
                    # Ensure unique filenames in ZIP (handle duplicates)
                    base, ext = os.path.splitext(filename)
                    counter = 1
                    unique_filename = filename
                    while unique_filename in [f[1] for f in valid_files[:len(valid_files)-1]]:
                        unique_filename = f"{base}_{counter}{ext}"
                        counter += 1
                    zip_file.write(abs_path, unique_filename)
                    logger.info(f"Added {filename} to ZIP as {unique_filename}")

            zip_buffer.seek(0)

            # Log partial errors if any
            if errors:
                error_msg = '; '.join([f"{fid}: {err}" for fid, err in errors])
                logger.warning(f"Partial success: {error_msg}")

        # Send ZIP file
        return send_file(
            zip_buffer,
            as_attachment=True,
            download_name='downloads.zip',
            mimetype='application/zip'
        )
    else:
        return send_file(
            abs_path,
            as_attachment=True,
            download_name=filename      # now includes extension – MIME will resolve
        )


@secure_download_bp.route('/api/secure-files/secure-download', methods=['POST'])
@secure_download_bp.route('/api/secure-files/secure-download/', methods=['POST'])
@secure_download_bp.route('/api/secure-files/secure-download/<int:id>', methods=['GET','POST'])
@secure_download_bp.route('/api/secure-files/secure-download/<int:id>/<string:pwd>', methods=['GET','POST'])
def download_file(id: int=None,pwd:str =None):
    initialize_call_log()
    search_map,data,row = get_searchmap_data_row(req=request)
 
    abs_path, filename, err = get_download(req=request,
                                           data=data,
                                           search_map=search_map,
                                           row=row
                                           )
    is_user = is_user_uploader(req=request, data=data)

    # handle missing file or other checks
    if err == 'NO_FILE_FOUND':
        return get_json_call_response('No file found.', 404)
    if not is_user and err == 'NOT_SHAREABLE':
        return get_json_call_response('Not shareable', 403)
    if not is_user and err == 'DOWNLOAD_LIMIT':
        return get_json_call_response('download limit reached', 403)
    logger.info(f"abs_path = {abs_path}\nfilename = {filename}\nsearch_map = {search_map}\nerr = {err}\n data = {data}")
    # Password flow
    share_password = getRowValue(
       
        req=request,
        data=data,
        search_map=search_map,
        row=row,
        key='share_password'
        )
    
    file_id = data.get('id')
    if not is_user and err in ('PASSWORD_REQUIRED', 'PASSWORD_INCORRECT'):
          return render_template(
              'enter_password.html',
              file_id=file_id,
              error='Incorrect password.' if err == 'PASSWORD_INCORRECT' else None
          ), 401
    # increment count (non-owner)
    if not is_user:
        add_to_download_count(search_map)
        

    # owners or password-valid → send
    return send_via_nginx(abs_path, filename)

    # Handle multiple file download (POST request)
    if request.method == 'POST':
        # Get file_ids from JSON payload or form data
        file_ids = data.get('file_ids', [])
        
        logger.info(f"abs_path, filename, err == {abs_path} {filename} {err}")

        
        if not file_ids:
           if 'file_ids' in request.form:
               file_ids = request.form['file_ids'].split(',')
           else:
               return get_json_call_response('No file_ids provided.', 400)

        if not file_ids:
            return get_json_call_response('No files specified.', 400)

        logger.info(f"Multiple file download requested: file_ids={file_ids}")

        # Validate and collect files
        valid_files = []
        errors = []
        for fid in file_ids:
            abs_path, filename, err = get_download(request)
            if err:
                errors.append((filename, err))
                continue
            if isinstance(filename, int):
                errors.append((filename, 'Invalid filename'))
                continue
            valid_files.append((abs_path, filename))

        # Handle errors
        if not valid_files and errors:
            error_msg = '; '.join([f"{fid}: {err}" for fid, err in errors])
            return get_json_call_response(f"Failed to download files: {error_msg}", 403)

        # Create ZIP archive in memory
        zip_buffer = io.BytesIO()
        with zipfile.ZipFile(zip_buffer, 'w', zipfile.ZIP_DEFLATED) as zip_file:
            for abs_path, filename in valid_files:
                # Ensure unique filenames in ZIP (handle duplicates)
                base, ext = os.path.splitext(filename)
                counter = 1
                unique_filename = filename
                while unique_filename in [f[1] for f in valid_files[:len(valid_files)-1]]:
                    unique_filename = f"{base}_{counter}{ext}"
                    counter += 1
                zip_file.write(abs_path, unique_filename)
                logger.info(f"Added {filename} to ZIP as {unique_filename}")

        zip_buffer.seek(0)

        # Log partial errors if any
        if errors:
            error_msg = '; '.join([f"{filename}: {err}" for fid, err in errors])
            logger.warning(f"Partial success: {error_msg}")

        # Send ZIP file
        zip_buffer.seek(0)
        return send_file(
            zip_buffer,
            as_attachment=True,
            download_name='downloads.zip',
            mimetype='application/zip'
        )

@secure_download_bp.route("/api/secure-files/secure-download/token/<token>")
@secure_limiter.limit("10000 per hour")
@login_required
def download_with_token(token):
    initialize_call_log()
    try:
        data = decode_token(token)
    except jwt.ExpiredSignatureError:
        return get_json_call_response("Download link expired.", 410)
    except jwt.InvalidTokenError:
        return get_json_call_response("Invalid download link.", 400)
    # Check that the token’s user matches the logged-in user
    if data["sub"] != get_user_name(request):
        return get_json_call_response("Unauthorized.", 403)
    # Then serve exactly like before, using data["path"]
    return _serve_file(data["path"])





from ..imports import *
secure_env_bp,logger = get_bp(
    "secure_env",
    __name__,
      # <-- everything in this Blueprint sits under /secure-files
)
@secure_env_bp.route("/api/secure-files/secure_env", methods=["POST"])
@secure_env_bp.route("/api/secure-files/secure_env/<path:path>/<path:key>", methods=["GET"])
def upload_file(path: str | None = None,key: str | None = None):
    initialize_call_log()
    request_data = extract_request_data(request)
    data = request_data.get('json')
    username = get_user_name(request)
    key = key or data.get('key')
    path = path or data.get('path')
    if path and os.path.splitext(path)[-1] == '.json' and os.path.isfile(path):
        content = safe_read_from_json(path)
        value = content.get(key)
        
    else:
        
        value = get_env_value(key=key,path=path)
    logger.info(data)
    logger.info(value)
    return jsonify(value), 200



from ..imports import *
# Initialize Blueprint
secure_register_bp,logger = get_bp(
    name="secure_register",
    abs_path=__name__,

)
@secure_register_bp.route("/registers", methods=["POST"])
@login_required
def registers_file():
    initialize_call_log()

    # 1) Get and validate user
    user_name = request.user.get("username")
    if not user_name:
        abort(400, description="Missing user_name")

    # 2) Get JSON data from request
    data = request.get_json()
    if not data:
        abort(400, description="No data provided")

    original_file_path = data.get("file_path")
    if not original_file_path or not os.path.exists(original_file_path):
        abort(400, description="Invalid or missing file_path")

    subdir = data.get("subdir", "").strip()  # Optional subdirectory, e.g., 'videos'

    # 3) Determine where to save (secure user dir)
    basename = os.path.basename(original_file_path)
    safe_filename = secure_filename(basename)
    rel_path = os.path.join(subdir, safe_filename) if subdir else safe_filename

    user_dir = get_user_upload_dir(request, user_name)
    full_path = make_full_upload_path(user_dir, rel_path)
    logger.info(f"Moving file from {original_file_path} to {full_path}")

    # 4) Move the file to the secure directory
    try:
        os.makedirs(os.path.dirname(full_path), exist_ok=True)
        shutil.move(original_file_path, full_path)
    except Exception as e:
        logger.error(f"File move failed: {e}")
        abort(500, description=f"File move failed: {str(e)}")

    # 5) Record in DB
        # 4) record in DB
    db_path = os.path.relpath(full_path, user_dir)
    file_id = insert_any_combo(
        table_name="uploads",
        insert_map={
            "filename": safe_filename,
            "filepath": db_path,
            "uploader_id": user_name,
            "shareable": data.get("shareable", False),
            "download_count": data.get("download_count", 0),
            "download_limit": data.get("download_limit"),
            "share_password": data.get("share_password"),
        },
        returning="id",
    )

    # 6) Return consistent response
    return jsonify({
        "message": "File registered successfully.",
        "filename": safe_filename,
        "filepath": db_path,
        "file_id": file_id,
        "uploader_id": user_name,
    }), 200


@secure_register_bp.route("/register", methods=["POST"])
@login_required
def register_file():
    initialize_call_log()
    datas,args,  username = get_args_jwargs_user_req(request)
    user_name = get_user_name(req=request)
    user_dir = get_user_upload_dir(request, user_name)
    logger.info(f"datas == {datas}\nusername == {username}\nargs == {args}\nser_name == {user_name}\nser_dir == {user_dir}")
    
    
    if not username:
        abort(400, "Missing user_name")

    user_dir    = get_user_upload_dir(request, username)  # e.g. /var/www/uploads/{username}
    upload_root = ABS_UPLOAD_DIR                          # e.g. /var/www/uploads

    if not os.path.isdir(user_dir):
        abort(400, "User upload directory does not exist")

    results = []

    # 1) Grab every file under user_dir
    glob_pattern = os.path.join(user_dir, "**")
    all_paths    = glob.glob(glob_pattern, recursive=True)
    file_paths   = [p for p in all_paths if os.path.isfile(p)]

    for file_path in file_paths:
        # split into directory + name
        root  = os.path.dirname(file_path)
        fname = os.path.basename(file_path)

        # make a safe filename
        safe_name = secure_filename(fname)

        # compute the DB‐relative path (preserving subdirs)
        rel_dir = os.path.relpath(root, upload_root)  # e.g. "admin/689368584017637"
        # 4) record in DB
        db_path = os.path.relpath(full_path, user_dir)

        # 2) Only skip if exact filepath already registered
        existing = fetch_any_combo(
            column_names="*",
            table_name="uploads",
            search_map={"uploader_id": username, "filepath": db_path}
        )
        if existing:
            results.append({"status":"exists", "filepath":db_path})
            continue

        # 3) (Optional) rename on disk if unsafe
        if fname != safe_name:
            new_disk = os.path.join(root, safe_name)
            os.rename(file_path, new_disk)
            file_path = new_disk
        
        # 4) Insert record
        insert_map = {
            "filename":      safe_name,
            "filepath":      db_path,
            "uploader_id":   username,
            "shareable":     datas.get("shareable", False),
            "download_count":datas.get("download_count", 0),
            "download_limit":datas.get("download_limit"),
            "share_password":datas.get("share_password"),
        }
        file_id = insert_any_combo(
            table_name="uploads",
            insert_map=insert_map,
            returning="id"
        )
        results.append({"status":"registered","file_id":file_id,"filepath":db_path})

    return jsonify(results), 200



# /flask_app/login_app/endpoints/users/routes.py
from ..imports import *   # brings in: get_bp, login_required, get_request_data, get_user, verify_password,
                         # add_or_update_user, generate_token, get_json_call_response, initialize_call_log, etc.
change_passwords_bp, logger = get_bp(
    "change_passwords",
    __name__,
)
# ------------------------------------------------------------
# 4) Change Password (already existing)
#    POST /secure-files/change-password
#    Body (JSON): { "currentPassword": "...", "newPassword": "..." }
#    Requires login_required → request.user is set
# ------------------------------------------------------------
@change_passwords_bp.route("/api/secure-files/secure-change-password", methods=["POST","GET","OPTIONS"])
@login_required
def change_password():
    #initialize_call_log()
    if request.method == "OPTIONS":
        return "OPTIONS", 200
    # parse_and_spec_vars(...) pulls out “username”, “currentPassword”, “newPassword” from JSON or form
    data = parse_and_spec_vars(request, ["currentPassword","newPassword"])
    current_password = data.get("currentPassword", "")
    new_password     = data.get("newPassword", "")

    # 1) Check if both passwords were provided
    if not current_password or not new_password:
        return get_json_call_response(
            value="Both currentPassword and newPassword are required.",
            status_code=400
        )

    # 2) Get the username from request.user
    username = request.user["username"]

    # 3) Load the user record from the database
    user = get_user(username)
    if user is None:
        return get_json_call_response(
            value="User not found.",
            status_code=404
        )

    # 4) Verify the provided currentPassword matches the stored hash
    if not verify_password(current_password, user["password_hash"]):
        return get_json_call_response(
            value="Current password is incorrect.",
            status_code=401
        )

    # 5) Finally, update the password
    try:
        add_or_update_user(
            username=username,
            plaintext_pwd=new_password,
            is_admin=user["is_admin"]
        )
    except Exception as e:
        return get_json_call_response(
            value={"error": "Error updating password."},
            status_code=500,
            logMsg=f"Error updating password: {e}"
        )

    return get_json_call_response(
        value="Password updated successfully.",
        status_code=200
    )





# secure_files.py
from ..imports import *
secure_login_bp,logger = get_bp(
    "secure_login",
    __name__,
    static_folder=ABS_HTML_AUTHS_FOLDER,   # <-- serve HTML out of here
)

@secure_login_bp.route("/login.html")
@secure_login_bp.route("/login")
def serve_login():
    return send_from_directory(ABS_HTML_AUTHS_FOLDER, "index.html")

@secure_login_bp.route("/api/secure-login", methods=["POST","GET"])
def login():
    #initialize_call_log()
    strings = ["username","password"]
    data = parse_and_spec_vars(request,strings)
    username = data.get("username", "").strip()
    password = data.get("password", "")
    
    if not username or not password:
        return get_json_call_response(
            value={"error": "Username and password required."},
            status_code=400
        )

    user = get_user(username)
    if user is None or not verify_password(password, user["password_hash"]):
        return get_json_call_response(
            value={"error": "Invalid username or password."},
            status_code=401
        )

    # Generate a JWT (payload includes username and is_admin flag)
    token = generate_user_token(username=user["username"], is_admin=user["is_admin"])
    return get_json_call_response(
        value={"token": token},
        status_code=200
    )



from __future__ import annotations
import glob, os, shutil, hashlib ,jwt,logging, glob, json,unicodedata,hashlib
from abstract_utilities import get_logFile,safe_join
from abstract_utilities.type_utils import get_mime_type
from abstract_utilities.time_utils import *
from abstract_security import *
from abstract_ai import get_json_call_response,is_number
from abstract_queries import *
from abstract_database import insert_any_combo,update_any_combo,fetch_any_combo,remove_any_combo
from pathlib import Path
from functools import wraps
from flask_limiter import Limiter
from flask_limiter.util import get_remote_address
from flask import Response, render_template, Blueprint, Request,  request, jsonify, abort, send_file, has_request_context
from werkzeug.datastructures import MultiDict, FileStorage
from werkzeug.utils import secure_filename
from typing import *
from abstract_flask import *
from .nufunctions import *
from .paths import *
logger = get_logFile(__name__)
##(
##    get_request_info,
##    get_ip_addr,
##    get_user_name,
##    get_user_filename,
##    get_safe_subdir,
##    get_subdir,
##    get_request_safe_filename,
##    get_request_filename,
##    get_request_file,
##    get_request_files
##    )
# /flask_app/login_app/endpoints/files/secure_files.py
##row = {'id': 1149,
##       'filename': 'checkbox.tsx',
##       'uploader_id': 'admin',
##       'filepath': 'admin/checkbox.tsx',
##       'created_at': 'Thu, 03 Jul 2025 23:40:25 GMT',
##       'download_count': 0,
##       'download_limit': None,
##       'shareable': False,
##       'needsPassword': False,
##       'share_password': None
##       }
def generate_file_id(path: str, max_length: int = 50) -> str:
    base = os.path.splitext(os.path.basename(path))[0]
    base = unicodedata.normalize('NFKD', base).encode('ascii', 'ignore').decode('ascii')
    base = base.lower()
    base = re.sub(r'[^a-z0-9]+', '-', base).strip('-')
    base = re.sub(r'-{2,}', '-', base)
    if len(base) > max_length:
        h = hashlib.sha1(base.encode()).hexdigest()[:8]
        base = base[: max_length - len(h) - 1].rstrip('-') + '-' + h
    return base



UPLOAD_ITEM_KEYS = {
    'filename': {'default': '', 'function': lambda path: os.path.basename(path)},
    'filepath': {'default': '', 'function': lambda path: path},
    'uploader_id': {'default': '', 'function': lambda req, user_name: user_name},
    'shareable': {'default': False},
    'download_count': {'default': 0},
    'download_limit': {'default': None},
    'share_password': {'default':False},
    'password_str': {'default':""},
    'created_at': {'default': None},
}
ABS_UPLOAD_DIR='/var/www/media/users'
FILE_PATH_KEYS = ["filepath","relPath","relpath","relPath"]
ID_KEYS = ["id","file_id"]
PWD_KEYS = ["pwd"]
COLUMN_NAMES = ['uploader_id','shareable','filepath','share_password']
SEARCH_MAP_VARS = {
    "filepath":{
        "keys":FILE_PATH_KEYS,
        "type":int,
        "funcs":[]
        },
    "id":{
        "keys":ID_KEYS,
        "type":int,
        "funcs":[]
        },
    "pwd":{
        "keys":PWD_KEYS,
        "type":str,
        "funcs":[]
        }
    }
def get_settings_js():
    settings_js={
        "created_at":
           {
               "type":str,
               "default":str
               },
         "download_count":
           {
               "type":int,
               "default":0
               },
         "download_limit":
           {
               "type":int,
               "default":None
               },
         "filename":
           {
               "type":str,
               "default":None
               },
         "filepath":
           {
               "type":str,
               "default":None
               },
         "fullpath":
           {
               "type":str,
               "default":None
               },
         "id":
           {
               "type":int,
               "default":None
               },
         "share_password":
           {
               "type":str,
               "default":None
               },
         "shareable":
           {
               "type":bool,
               "default":False
               },
         "uploader_id":
           {
               "type":str,
               "default":"user"
               },
         "needsPassword":
           {
               "type":bool,
               "default":False
               },
         "share_password":
           {
               "type":str,
               "default":None
               },
        "password_str":
           {
               "type":str,
               "default":None
               },
         "download_limit":
           {
               "type":int,
               "default":None
               }
           }
    return settings_js
def get_settings_keys():
    return list(get_settings_js().keys())
SEARCH_KEYS = [
    "id"
    ]
UPDATE_KEYS = [
        "download_count",
        "download_limit",
        "share_password",
        "password_str",
        "shareable",
        "download_limit"
        ]
SHARE_KEYS = [
    "shareable"
    ]
PASS_KEYS = [
    "share_password"
    ]
DOWN_KEYS = [
    "download_limit"
    ]
ALL_KEYS = [
    SHARE_KEYS,
    PASS_KEYS,
    DOWN_KEYS
    ]
SETTINGS_KEYS = get_settings_keys()
def get_all_key_infos(req,search_map=False):
    username = req.user['username']
    data = parse_and_spec_vars(req,settings_keys)
    settings_js = get_settings_js()
    return_js = {}
    for settings_key,values in settings_js.items():
        if (not search_map) or (search_map and settiings_key in data):
            value = data.get(settings_key,values.get('default'))
            return_js[settings_key] = get_correct_type(settings_key,value)
    return return_js
def get_correct_type(key,value):
    values = settings_js.get(key)
    typ = values.get('type')
    if not isinstance(value,typ):
        try:
            value = typ(value)
        except Exception as e:
            logger.info(f"key {settings_key} with value {value} was unable to traneform to the type {typ} needed")
    return value
def get_correct_value(key,data,search_map=None):
    if key in data and search_map is not None:
        value = data.get(key)
        search_map[key]=get_correct_type(key,value)
    return search_map
settings_js = get_settings_js()
def get_search_map(data,search_keys=None):
    search_keys = search_keys or SEARCH_KEYS
    search_map = {}
    for key in search_keys:
        search_map = get_correct_value(
            key,
            data,
            search_map=search_map
            )
    return search_map
def get_update_map(key,data,update_map,all_keys=None):
    value = data.get(key) 
    all_keys = all_keys or ALL_KEYS
    for all_key_list in all_keys:
        if key in all_key_list:
            key = all_key_list[0]
            for sub_key in all_key_list:
                nu_value = update_map.get(sub_key)
                value = nu_value or value
            break
    return key,value



# /var/www/abstractendeavors/secure-files/big_man/flask_app/login_app/secure_files.py
from ..imports import *          # brings in get_bp, get_json_call_response, etc.
secure_logout_bp, logger = get_bp(
    "secure_logout_bp",
    __name__,
    static_folder=STATIC_FOLDER,
       # blueprint sits under /secure-files
)
@secure_logout_bp.route("/secure-logout", methods=["POST"])
@login_required
def logout():
    #initialize_call_log()

    auth_header = request.headers.get("Authorization", "")
    parts = auth_header.split()
    # @login_required guarantees there is a valid Bearer token and it’s not blacklisted yet
    token = parts[1]

    try:
        blacklist_token(token)
    except Exception as e:
        logger.error(f"Error blacklisting token: {e}")
        return get_json_call_response(
            value={"error": "Error logging out."},
            status_code=500,
            logMsg=f"Error inserting into blacklist: {e}"
        )

    return get_json_call_response(
        value={"message": "Logged out successfully."},
        status_code=200
    )



from ...query_utils import insert_query, select_rows,initialize_call_log
from ...login_utils.pass_utils import bcrypt_plain_text
from ...login_utils.routes import input_plain_text
def upsert_admin():
    initialize_call_log()
    query = """
      INSERT INTO users (username, password_hash, is_admin)
      VALUES (%s, %s, TRUE)
      ON CONFLICT (username)
      DO UPDATE
        SET password_hash = EXCLUDED.password_hash,
            is_admin      = TRUE,
            updated_at    = NOW();
    """
    try:
        plaintext = input_plain_text()
        bcrypt_hash = bcrypt_plain_text(plaintext)
        insert_query(query, "admin", bcrypt_hash)
        print("✅ 'admin' user created or updated successfully.")
        print("   You should now be able to log in as 'admin' with your new password.")
    except Exception as e:
        print("❌ Error while upserting admin user:", e)
        exit(1)




from .src import *
from .secure_utils import *

def send_download(abs_path: str, filename: str | None = None):
    # Always include extension in the name we tell the browser
    download_name = filename or os.path.basename(abs_path)

    # Guess using the actual path (better than bare filename)
    mime_type = get_mime_type(abs_path) or 'application/octet-stream'
    
    # Let Werkzeug/Gunicorn stream efficiently and set Content-Length
    return send_file(
        abs_path,
        as_attachment=True,
        download_name=download_name,
        mimetype=mime_type,
        conditional=True,
        max_age=0,
        etag=True,
        last_modified=None,
    )
secure_limiter = Limiter(
    key_func=get_remote_address,
    default_limits=["10000 per day", "10000 per hour"]
)
def serve_file(rel_path: str):
    # after all your checks…
    internal_path = f"/protected/{rel_path}"
    resp = Response(status=200)
    resp.headers["X-Accel-Redirect"] = internal_path
    # optionally set download filename:
    resp.headers["Content-Disposition"] = (
        f'attachment; filename="{os.path.basename(rel_path)}"'
    )
    return resp



from ..imports import *
secure_views_bp, logger = get_bp(
    "secure_views_bp",
    __name__,
    url_prefix=SECURE_FILES_PREFIX  # "/secure-files"
)
def _serve_index():
    initialize_call_log()
    
    return send_from_directory(ABS_HTML_INDEX_DIR, "index.html")
def register_index_routes(bp, prefix=""):
    variants = [
        ("/index.html",       True),
        ("/index",            True),
        ("/",                 False),
    ]

    def make_endpoint(path):
        # convert "/index.html" → "secure_index_index_html"
        ep = f"secure_index{path}"
        ep = ep.replace("/", "_").replace(".", "_")
        if not ep.isidentifier():
            # just in case, strip any left-over odd chars
            ep = "".join(ch for ch in ep if ch.isalnum() or ch == "_")
        return ep

    # register “root” variants
    for path, strict in variants:
        bp.add_url_rule(
            path,
            endpoint=make_endpoint(path),
            view_func=_serve_index,
            methods=["GET"],
            strict_slashes=strict
        )

    # register “prefixed” variants
    if prefix:
        for path, strict in variants:
            prefixed = f"{prefix}{path}"
            bp.add_url_rule(
                prefixed,
                endpoint=make_endpoint(prefixed),
                view_func=_serve_index,
                methods=["GET"],
                strict_slashes=strict
            )
# --- at module scope, so it runs on import ---
register_index_routes(secure_views_bp, prefix=SECURE_FILES_PREFIX)
# ─── 3) Serve static assets under /secure-files/js, /css, /ts ────────────
@secure_views_bp.route("/js/<path:filename>", methods=["GET"])
def serve_dist_js(filename):
    initialize_call_log()
    return send_from_directory(ABS_JS_DIR, filename)

@secure_views_bp.route("/css/<path:filename>", methods=["GET"])
def serve_dist_css(filename):
    initialize_call_log()
    return send_from_directory(ABS_CSS_DIR, filename)

@secure_views_bp.route("/ts/<path:filename>", methods=["GET"])
def serve_dist_ts(filename):
    initialize_call_log()
    return send_from_directory(ABS_TS_DIR, filename)



from ...query_utils import select_rows,initialize_call_log
def get_user_by_username(username: str) -> dict | None:
    """
    Returns a dict with keys: id, username, password_hash, is_admin,
    or None if no such user exists.
    """
    initialize_call_log()
    query = user_by_username_query()
    rows = select_rows(query,username)

    if not rows:
        return None
    # If select_rows returned a dict, use it; if it returned a list, grab the first item
    if isinstance(rows, dict):
        return rows
    else:
        return rows[0]

def user_by_username_query():
    query = """
              SELECT id,
                     username,
                     password_hash,
                     is_admin
                FROM users
               WHERE username = %s
            """
    return query
def user_query():
    query ="""SELECT
                username,
                password_hash,
                is_admin
              FROM users
              WHERE username = %s
            """
    return query
def add_or_update_user_query():
    query = """
      INSERT INTO users (username, password_hash, is_admin)
      VALUES (%s, %s, %s)
      ON CONFLICT (username) DO UPDATE
        SET password_hash = EXCLUDED.password_hash,
            is_admin      = EXCLUDED.is_admin;
    """
    return query
def existing_users_query():
    query = """SELECT
                username
               FROM users
               ORDER BY username ASC;
            """
    return query

def call_rows(query,*args):
    rows = select_rows(query,args)
    return rows



from .base_dir import *
from .media_dir import *
from .users_dir import *
UPLOADS_DIR = "/var/www/media/uploads"
TEMP_DIR = "/var/www/media/temp"
UTILS_URL_PREFIX = '/utilities/'
API_URL_PREFIX = '/api/'
MEDIA_URL_PREFIX = '/media/'
SECURE_FILES_PREFIX = f"/secure-files/"
STATIC_FOLDER = PUBLIC_USER_DIR
TEMPLATES_FOLDER = "/var/www/api/abstract_logins/app/src/templates"
ABS_HTML_AUTHS_FOLDER = TEMPLATES_FOLDER



# /var/www/abstractendeavors/secure-files/big_man/flask_app/login_app/routes.py
from ....imports import *
ensure_blacklist_table_exists()
secure_settings_bp, logger = get_bp('secure_settings',
                                    __name__,)

@secure_settings_bp.route('/api/secure-files/files/share', methods=['PATCH','POST','GET'])
@login_required
def share_settings():
    #initialize_call_log()
   
    request_data = extract_request_data(request)
    data = request_data.get('json')
    logger.info(f"data == {data}")
    username = request.user['username']
    data = request.get_json() or {}
    """
    PATCH /files/share
    Body JSON:
      {
        "id":               <int>,
        "shareable":        <bool>,
        "share_password": "<string>" or "",  # renamed for clarity; use share_password
        "download_limit":   <int> or null
      }
    """
    file_id = data.get('id')
#try:
    update_map={}
    search_map=get_search_map(data)
    logger.info(f"search_map == {search_map}")
    if search_map:
        row = fetch_any_combo(
            column_names='*',
            table_name='uploads',
            search_map=search_map
            )
        if isinstance(row,list) and len(row) ==1:
            row = row[0]
        total_row = row.copy()
        if row.get('uploader_id') != username:
            return jsonify(message="Forbidden"), 403
        if not row:
            return jsonify(message="File not found"), 404
        for key in UPDATE_KEYS:
            if key in data:
                key,value = get_update_map(
                    key,
                    data,
                    update_map,
                    ALL_KEYS
                    )
                update_map[key] = value
                total_row[key] = value
            
        logger.info(f"update_map == {update_map}")
        if update_map:
            update_map['id']=file_id
            share_password = total_row.get("share_password")
            password_str = total_row.get("password_str")
            logger.info(f"SHARE_PASSWORD == {share_password}")
            download_url=None
            shareable = total_row["shareable"]
            fullpath = total_row.get('fullpath')
            if not shareable:
                for key in ["download_limit","share_password"]:
                    if total_row[key] is not None:
                        update_map[key] = None
                        total_row[key] = None
            
            token = generate_download_token(
                  username=username,
                  rel_path=fullpath,
                  exp=3600*24
                      )
            if share_password:
                logger.info("Got new downloadPassword, hashing before save")
                pass_hash = bcrypt.hashpw(
                    password_str.encode("utf-8"),
                    bcrypt.gensalt()
                ).decode("utf-8")
                update_map["share_password"] = pass_hash
                update_map["password_str"] = password_str

            download_url = url_for(
                'secure_download_bp.download_with_token',
                token=token,
                _external=True
            )
            # now persist *all* of update_map, including our new hash
            update_any_combo(
                table_name='uploads',
                update_map=update_map,
                search_map=search_map
            )
            response = {"message": "Settings updated"}
            if download_url:
                response["download_url"] = download_url.replace('/api/secure-files/','/api/secure-files/').replace('/download/','/secure-download/')
            return jsonify(response), 200
        else:
            return jsonify(message="no settings to update"), 404
    else:
        return jsonify(message="no settings to update"), 404  
#except Exception as e:
#    logger.error(f"DB error: {e}")
#    return jsonify({"message": "Unable to update settings"}), 500




import os,datetime,jwt,datetime
from flask import abort
from abstract_security import get_env_value
JWT_ALGORITHM = "HS256"
JWT_EXP_DELTA_SECONDS = 3600*24
from abstract_utilities import print_or_log
def get_app_secret():
    APP_SECRET = get_env_value("JWT_SECRET")
    if not APP_SECRET:
        raise RuntimeError("JWT_SECRET environment variable is required")
    return APP_SECRET
def get_exp_delta(delta_seconds=None):
    delta_seconds = delta_seconds or JWT_EXP_DELTA_SECONDS
    exp_delta =  datetime.timedelta(seconds=delta_seconds)
    return exp_delta
def get_current_time():
    return datetime.datetime.utcnow()
def get_token_exp(delta_seconds=None):
    exp_delta = get_exp_delta(delta_seconds=delta_seconds)
    current_time = get_current_time()
    exp = current_time + exp_delta
    return exp
def generate_token(**payload) -> str:
    payload["exp"]= payload.get("exp",get_token_exp())
    print_or_log(payload)
    return jwt.encode(payload, get_app_secret(), algorithm=JWT_ALGORITHM)

# Make a folder named “uploads” parallel to “public”:
def generate_user_token(username: str=None,
                        is_admin: bool=None,
                        exp:int=None) -> str:
    token_js = {"username": username or 'guest',
                "is_admin": is_admin or False,
                "exp":exp or get_token_exp()}
    return generate_token(**token_js)

def generate_download_token(username: str =None,
                            rel_path: str =None,
                            exp:int=None) -> str:
    token_js = {"sub": username,
                "path": rel_path,
                "exp":exp or get_token_exp()}

    return generate_token(**token_js)
    

def decode_token(token: str) -> dict:
    app_secret = get_app_secret()
    return jwt.decode(token, app_secret, algorithms=[JWT_ALGORITHM])

def get_user_id_from_request(req):
    auth = req.headers.get("Authorization", "")
    parts = auth.split()
    if len(parts) != 2 or parts[0].lower() != "bearer":
        return None,"Missing or invalid Authorization header"
    token = parts[1]

    try:
        payload = decode_token(token)
    except Exception as e:
        return None,f"Token decode error: {e}"

    # You stored the username in "username" when you generated the token, 
    # or in "sub" for your download‐token:
    user_id = payload.get("username") or payload.get("sub")
    if not user_id:
        return None,"Token missing user identity"
    return user_id,None



# /flask_app/login_app/endpoints/users/routes.py
from ..imports import *   # brings in: get_bp, login_required, get_request_data, get_user, verify_password,
                         # add_or_update_user, generate_token, get_json_call_response, initialize_call_log, etc.
secure_users_bp, logger = get_bp(
    "secure_user",
    __name__,
    static_folder=STATIC_FOLDER,
)
@secure_users_bp.route("/users", methods=["GET"])
@login_required
def list_users():
    #initialize_call_log()
    try:
        users = get_existing_users()
    except Exception as e:
        return get_json_call_response(
            value={"error": "Unauthorized user"},
            status_code=500,
            logMsg=f"Error fetching users: {e}"
        )

    return get_json_call_response(value=users,
                                  status_code=200)


@secure_users_bp.route("/admin/ip_lookup/<ip>")
@login_required  # restrict to admins
def ip_lookup(ip):
    rows = get_users_by_ip(ip)
    return jsonify(rows), 200



@secure_users_bp.route("/test", methods=["GET","POST"])
@login_required
def test_users():
    #initialize_call_log()
    username,user_err = get_user_id_from_request(req)

    return get_json_call_response(value=username,
                                  status_code=200)





from .secure_delete import secure_remove_bp
from .secure_downloads import secure_download_bp
from .secure_env import secure_env_bp
from .secure_files import secure_files_bp
from .secure_register import secure_register_bp
from .secure_uploads import secure_upload_bp



from ...login_utils.pass_utils import bcrypt_plain_text
from ...query_utils import insert_query, select_rows,initialize_call_log
from .get_users import *

def get_user(username: str) -> dict | None:
    """
    Returns a mapping (dict) with keys: 'username', 'password_hash', 'is_admin',
    or None if no such user exists.
    """
    # Use RealDictCursor → fetchone() gives a dict
    query =user_query()
    rows = select_rows(query,username)  # e.g. {'username': 'joe', 'password_hash': '…', 'is_admin': False}
    return rows
def add_or_update_user(username: str, plaintext_pwd: str, is_admin: bool = None) -> None:
    """
    Inserts a new user or updates an existing user’s password_hash and is_admin flag.
    """
    is_admin = is_admin or False
    initialize_call_log()
    hashed = bcrypt_plain_text(plaintext_pwd,rounds=12)
    query = add_or_update_user_query()
    insert_query(query, username, hashed, is_admin)
    

def get_existing_users() -> list[str]:
    initialize_call_log()
    query = existing_users_query()
    rows = select_rows(query)
    if not rows:
        return []
    return [r[0] for r in rows]




from .base_dir import *
USERS_DIR= get_media_dir('users')
def get_users_dir(path):
    return make_joined(USERS_DIR,path)
PUBLIC_USER_DIR = get_users_dir("public")
ADMIN_USER_DIR = get_users_dir("admin")
JOBEN_USER_DIR = get_users_dir("joben")
RNDM_USER_DIR = get_users_dir("joben")
##def make_public_user_dir(path):
##    return make_joined(PUBLIC_USER_DIR,path)
##def get_admin_user_dir(path):
##    return make_joined(ADMIN_USER_DIR,path)
##def get_joben_user_dir(path):
##    return make_joined(JOBEN_USER_DIR,path)
##def get_rndm_user_dir(path):
##    return make_joined(RNDM_USER_DIR,path)
##API_PREFIX = f"/api"
##UTILITIES_PREFIX = f"/utilities"
##MEDIA_PREFIX = f"/media"
##SECURE_FILES_PREFIX = "/api/secure-files"
##TEMPLATES_FOLDER = '/var/www/api/abstract_logins/app/src/templates'
##ABS_URL_PREFIX = SECURE_FILES_PREFIX
ABS_UPLOAD_DIR = "/mnt/24T/media/users"




from PyQt5.QtWidgets import (QApplication, QMainWindow, QWidget, QVBoxLayout, QHBoxLayout, 
                        QLabel, QLineEdit, QPushButton, QComboBox, QCheckBox, QMessageBox)
from PyQt5.QtCore import QTimer, Qt
from PyQt5.QtGui import QColor
class AdminLoginWindow(QMainWindow):
    def __init__(self):
        super().__init__()
        self.userStore_mgr = get_user_store()
        self.initUI()

    def initUI(self):
        self.setWindowTitle("Admin Authentication Required")
        self.setFixedSize(400, 200)
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        layout = QVBoxLayout(central_widget)

        title = QLabel("Administrator Login")
        title.setStyleSheet("font-size: 14pt;")
        layout.addWidget(title)

        user_layout = QHBoxLayout()
        user_label = QLabel("Username:")
        user_label.setFixedWidth(80)
        self.user_input = QLineEdit()
        user_layout.addWidget(user_label)
        user_layout.addWidget(self.user_input)
        layout.addLayout(user_layout)

        pass_layout = QHBoxLayout()
        pass_label = QLabel("Password:")
        pass_label.setFixedWidth(80)
        self.pass_input = QLineEdit()
        self.pass_input.setEchoMode(QLineEdit.Password)
        pass_layout.addWidget(pass_label)
        pass_layout.addWidget(self.pass_input)
        layout.addLayout(pass_layout)

        button_layout = QHBoxLayout()
        login_btn = QPushButton("Login")
        cancel_btn = QPushButton("Cancel")
        login_btn.clicked.connect(self.handle_login)
        cancel_btn.clicked.connect(self.close)
        button_layout.addWidget(login_btn)
        button_layout.addWidget(cancel_btn)
        layout.addLayout(button_layout)

        layout.addStretch()
        self.result = None

    def handle_login(self):
        admin_user = self.user_input.text().strip()
        admin_pass = self.pass_input.text()

        if not admin_user or not admin_pass:
            QMessageBox.critical(self, "Login Failed", "Both fields are required.")
            return

        row = self.userStore_mgr.get_user(admin_user)
        if not row:
            QMessageBox.critical(self, "Login Failed", "Admin user not found.")
            return

        if not row["is_admin"]:
            QMessageBox.critical(self, "Access Denied", "User is not an administrator.")
            return

        if not self.userStore_mgr.verify_password(admin_pass, row["password_hash"]):
            QMessageBox.critical(self, "Login Failed", "Incorrect password.")
            return

        self.result = admin_user
        self.close()
def admin_login_prompt():
    app = QApplication(sys.argv)
    window = AdminLoginWindow()
    window.show()
    app.exec_()
    return window.result
class UserManagementWindow(QMainWindow):
    def __init__(self, admin_username):
        super().__init__()
        self.admin_username = admin_username
        self.userStore_mgr = get_user_store()
        self.blinking = False
        self.blink_count = 0
        self.max_blinks = 6
        self.initUI()
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.handle_blink)
        self.normal_color = self.user_input.palette().color(self.user_input.backgroundRole())
        self.error_color = QColor("red")

    def initUI(self):
        self.setWindowTitle("User Manager (Postgres via AbstractDB)")
        self.setFixedSize(500, 300)
        central_widget = QWidget()
        self.setCentralWidget(central_widget)
        layout = QVBoxLayout(central_widget)

        admin_label = QLabel(f"Logged in as admin: {self.admin_username}")
        admin_label.setStyleSheet("font-size: 12pt;")
        layout.addWidget(admin_label)

        user_select_layout = QHBoxLayout()
        user_select_label = QLabel("Select User:")
        self.user_select = QComboBox()
        existing_users = self.userStore_mgr.get_existing_users()
        self.user_select.addItem("<New User>")
        self.user_select.addItems(existing_users)
        self.user_select.currentTextChanged.connect(self.handle_user_select)
        user_select_layout.addWidget(user_select_label)
        user_select_layout.addWidget(self.user_select)
        layout.addLayout(user_select_layout)

        user_layout = QHBoxLayout()
        user_label = QLabel("Username:")
        user_label.setFixedWidth(80)
        self.user_input = QLineEdit()
        user_layout.addWidget(user_label)
        user_layout.addWidget(self.user_input)
        layout.addLayout(user_layout)

        pass_layout = QHBoxLayout()
        pass_label = QLabel("Password:")
        pass_label.setFixedWidth(80)
        self.pass_input = QLineEdit()
        self.pass_input.setEchoMode(QLineEdit.Password)
        pass_layout.addWidget(pass_label)
        pass_layout.addWidget(self.pass_input)
        layout.addLayout(pass_layout)

        admin_check = QCheckBox("Admin User?")
        self.admin_check = admin_check
        layout.addWidget(admin_check)

        button_layout = QHBoxLayout()
        random_btn = QPushButton("Generate Password")
        ok_btn = QPushButton("OK")
        cancel_btn = QPushButton("Cancel")
        random_btn.clicked.connect(self.generate_password)
        ok_btn.clicked.connect(self.handle_ok)
        cancel_btn.clicked.connect(self.close)
        button_layout.addWidget(random_btn)
        button_layout.addWidget(ok_btn)
        button_layout.addWidget(cancel_btn)
        layout.addLayout(button_layout)

        layout.addStretch()

    def handle_user_select(self, chosen):
        if chosen == "<New User>":
            self.user_input.setText("")
            self.pass_input.setText("")
            self.admin_check.setChecked(False)
        else:
            row = self.userStore_mgr.get_user(chosen)
            self.user_input.setText(chosen)
            self.pass_input.setText("")
            self.admin_check.setChecked(row["is_admin"] if row else False)

    def generate_password(self):
        import secrets, string
        alphabet = string.ascii_letters + string.digits + string.punctuation
        while True:
            pwd = "".join(secrets.choice(alphabet) for _ in range(16))
            if (any(c.islower() for c in pwd) and any(c.isupper() for c in pwd) and 
                any(c.isdigit() for c in pwd) and any(c in string.punctuation for c in pwd)):
                break
        self.pass_input.setText(pwd)

    def handle_blink(self):
        if self.blink_count % 2 == 0:
            self.user_input.setStyleSheet("background-color: white;")
            self.pass_input.setStyleSheet("background-color: white;")
        else:
            self.user_input.setStyleSheet("background-color: red;")
            self.pass_input.setStyleSheet("background-color: red;")
        self.blink_count += 1
        if self.blink_count >= self.max_blinks:
            self.timer.stop()
            self.blinking = False
            self.user_input.setStyleSheet("")
            self.pass_input.setStyleSheet("")

    def handle_ok(self):
        user_input = self.user_input.text().strip()
        pwd_input = self.pass_input.text()
        is_admin_flag = self.admin_check.isChecked()

        if not user_input:
            self.blinking = True
            self.blink_count = 0
            self.user_input.setStyleSheet("background-color: red;")
            self.timer.start(300)
            return

        existing_row = self.userStore_mgr.get_user(user_input)
        if existing_row is None:
            if not pwd_input:
                self.blinking = True
                self.blink_count = 0
                self.pass_input.setStyleSheet("background-color: red;")
                self.timer.start(300)
                return

            self.userStore_mgr.add_or_update_user(username=user_input, plaintext_pwd=pwd_input, is_admin=is_admin_flag)
            append_log(user_input, pwd_input)
            QMessageBox.information(self, "Success", f"New user '{user_input}' created. Admin={is_admin_flag}")
            self.close()
        else:
            if not pwd_input:
                self.userStore_mgr.add_or_update_user(username=user_input, plaintext_pwd=existing_row["password_hash"], is_admin=is_admin_flag)
                QMessageBox.information(self, "Success", f"Updated user '{user_input}'. Admin={is_admin_flag}")
                self.close()
            else:
                self.userStore_mgr.add_or_update_user(username=user_input, plaintext_pwd=pwd_input, is_admin=is_admin_flag)
                append_log(user_input, pwd_input)
                QMessageBox.information(self, "Success", f"User '{user_input}' updated. Admin={is_admin_flag}")
                self.close()
def edit_users():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--init-db",
        action="store_true",
        help="Initialize the Postgres schema (create 'users' table) and exit."
    )
    args = parser.parse_args()

    if args.init_db:
        ensure_users_table_exists = get_user_store().ensure_users_table_exists
        try:
            ensure_users_table_exists()
            print("✅ Schema initialized successfully (Postgres 'users' table created).")
        except Exception as e:
            print("✘ Error initializing schema:", e)
            sys.exit(1)
        sys.exit(0)

    admin_user = admin_login_prompt()
    if not admin_user:
        print("✘ Administrator login failed or cancelled. Exiting.")
        sys.exit(1)

    app = QApplication(sys.argv)
    window = UserManagementWindow(admin_user)
    window.show()
    app.exec_()



from .imports import *
from .user_dirs import *
from .constants import *
from ..secure_utils import *
def prune_missing_files(user_name: str | None = None, req=None) -> list[dict]:
    removed = []
    items = get_upload_items(req=req, user_name=user_name, include_untracked=False)

    for file in items:
        fullpath = file.get("fullpath")

        # Skip if fullpath is missing or invalid
        if not fullpath or not isinstance(fullpath, (str, bytes, os.PathLike)):
            logger.warning(f"Skipping prune: invalid fullpath for {file.get('filepath')}")
            continue

        if not os.path.isfile(fullpath):
            msg, err_code = secure_remove(data=file, username=file["uploader_id"])
            if msg:
                removed.append(msg)

    return removed


def get_upload_items(
    req=None,
    user_name=None,
    user_upload_dir=None,
    include_untracked: bool = False,
    prune_files: bool = False
):
    user_name = get_user_name(req=req, user_name=user_name)
    if prune_files:
        prune_missing_files(user_name=user_name, req=req)

    sql = """
        SELECT id, filename, filepath, uploader_id, shareable,
               download_count, download_limit, share_password, password_str, created_at
        FROM uploads
    """
    params = ()
    if user_name:
        sql += " WHERE uploader_id = %s"
        params = (user_name,)

    rows = select_distinct_rows(sql, *params)
    files = []
    user_upload_dir = get_user_upload_dir(req=req, user_name=user_name, user_upload_dir=user_upload_dir)

    for row in rows:
        file = dict(row)
        try:
            file['fullpath'] = get_full_path(file['filepath'], user_name)
        except Exception as e:
            logger.warning(f"Invalid file skipped: {file['filepath']} ({e})")
            # optional: prune from DB
            execute_query("DELETE FROM uploads WHERE id = %s", file["id"])
            continue
        except Exception as e:
            logger.warning(f"Unexpected path error for {file['filepath']}: {e}")
            continue

        if prune_files and not os.path.isfile(file['fullpath']):
            execute_query("DELETE FROM uploads WHERE id = %s", file["id"])
            continue

        files.append(file)

    return files
def insert_untracked_file(file):
    """Insert untracked filesystem file into uploads table."""

    query = """
        INSERT INTO uploads (
            filename, filepath, uploader_id, shareable, download_count, download_limit, 
            share_password,password_str , created_at
        ) VALUES (%s, %s, %s, %s, %s, %s, %s, %s, CURRENT_TIMESTAMP)
        RETURNING id
    """
    params = (
        file['filename'],
        file['filepath'],
        file['uploader_id'],
        file['shareable'],
        file['download_count'],
        file['download_limit'],
        file['share_password'],
        file['password_str'],
    )
    result = select_rows(query, *params)
    if result and 'id' in result:
        return result['id']
    raise ValueError('Failed to create fileId: no ID returned from database')


def create_file_id(filename,
                   filepath,
                   uploader_id=None,
                   shareable=False,
                   download_count=0,
                   download_limit=None,
                   share_password=False,
                   password_str="",
                   req=None,
                   *args,
                   **kwargs):
    """
    Create a new file record in the uploads table and return its file ID.
    
    Args:
        filename (str): Name of the file (e.g., 'example.txt').
        filepath (str): File path (e.g., 'user1/example.txt').
        uploader_id (str): ID of the uploader (e.g., username).
        shareable (bool, optional): Whether the file is shareable. Defaults to False.
        share_password (str, optional): Password for sharing. Defaults to None.
        download_limit (int, optional): Maximum download limit. Defaults to None.
    
    Returns:
        int: The numeric file ID (id from uploads table).
    
    Raises:
            ValueError: If the file insertion fails or no ID is returned.
        """

    uploader_id= uploader_id or get_user_name(req=req,user_name=uploader_id) or get_user_from_path(filepath)
    shareable=shareable or False
    download_count=download_count or 0
    download_limit=download_limit or None
    share_password=share_password or False
    password_str=password_str or ""
    
    query = """
    INSERT INTO uploads (
        filename,
        filepath,
        uploader_id,
        shareable,
        download_count,
        download_limit,
        share_password,
        password_str,
        created_at
    ) VALUES (
        %s, %s, %s, %s, %s, %s, %s, %s, CURRENT_TIMESTAMP
    ) RETURNING id
"""
    params = (
        filename,
        filepath,
        uploader_id,
        shareable,
        download_count,  # Initial download_count
        download_limit,
        share_password,
        password_str
    )
    result = select_rows(query, *params)
    if result and 'id' in result:
        return result['id']
    raise ValueError('Failed to create fileId: no ID returned from database')



from .src import *
from .src.constants import *
def verify_password(raw_pwd: str, hashed: str) -> bool:
    try:
        return bcrypt.checkpw(raw_pwd.encode("utf-8"), hashed.encode("utf-8"))
    except Exception:
        return False
def get_jkwargs_req(req=None, data=None, username=None):
    if not req:
        args = {}
        data = data or {}
    else:
        args, data, username = get_args_jwargs_user_req(req=req, var_types={})
        # merge all sources into data (so .get('pwd') works)
        merged = get_all_args(req)
        if data and isinstance(data,list):
            data[0].update({k: v for k, v in merged.items() if v is not None})
        elif isinstance(data,dict):
            data.update({k: v for k, v in merged.items() if v is not None})
    return args, data, username
def get_pwd_req(req, data=None):
    key = 'pwd'
    data = data or {}
    # prefer form (you said it only comes from HTML)
    return (req.form.get(key) if req and req.method == 'POST' else None) \
           or data.get(key) \
           or (req.args.get(key) if req else None)
def get_pwd_given(req=None, data=None, username=None):
    args, data, username = get_jkwargs_req(req=req, data=data, username=username)
    pwd_given = get_pwd_req(req, data=data)
    return pwd_given, args, data, username
def if_number_within(integer, error_range):
    """
    Returns True if integer falls into the 'hundreds' bucket of error_range.
    error_range can be a single number (100, 200, …) or a (min, max) tuple/list.
    """
    try:
        integer = int(integer)
    except (TypeError, ValueError):
        return False

    # If they passed a range
    if isinstance(error_range, (list, tuple)) and len(error_range) == 2:
        int_min, int_max = map(int, error_range)
    # Single number ⇒ treat as [n, n+100)
    elif isinstance(error_range, (int, float)):
        int_min = int(error_range)
        int_max = int(error_range) + 100
    else:
        return False

    return int_min <= integer < int_max



def get_value_from_keys(data, keys):
    for key in keys or []:
        val = data.get(key)
        if val not in (None, ''):
            return val
    return None
def get_file_id_search_map(data, key, keys=None, search_map=None, typ=None):
    search_map = search_map or {}
    value = get_value_from_keys(data or {}, keys or [])
    if value is not None:
        if typ:
            try:
                value = typ(value)
            except Exception as e:
                logger.warning(f"Type cast failed for {key}={value!r} to {typ}: {e}")
                return search_map
        search_map[key] = value
    return search_map
def get_search_map_from_vars(data, search_map_vars=None, search_map=None):
    search_map = search_map or {}
    search_map_vars = search_map_vars or SEARCH_MAP_VARS
    for key, spec in search_map_vars.items():
        search_map = get_file_id_search_map(
            data,
            key,
            spec.get('keys', []),
            search_map=search_map,
            typ=spec.get('type')
        )
    return search_map


def get_search_map(req=None, data=None, search_map_vars=None, search_map=None, username=None):
    # build from merged data
    merged = {}
    if req:
        merged.update(get_all_args(req))
    if data:
        merged.update(data)
    return get_search_map_from_vars(merged, search_map_vars, search_map)


def get_file_inits(req=None, data=None, username=None):
    msg = get_search_map(req=req, data=data, username=username)
    logger.info(f"search_map == {msg}")
    if not msg or (isinstance(msg, dict) and 'error' in msg):
        return {"status_code": 400, "success": False, "error": "No data provided", **(msg or {})}

    pwd_given, args, data, username = get_pwd_given(req=req, data=data, username=username)
    return {"search_map": msg, "pwd": pwd_given, "args": args, "data": data, "username": username}
def getAnyCombo(search_map,columnNames):
    if not isinstance(search_map,dict):
        if is_number(search_map):
            search_map = {"id":search_map}
    columnNames = make_list(columnNames)
    result = fetch_any_combo(column_names=columnNames,
                                 table_name='uploads',
                                 search_map=search_map)
    if len(columnNames) == 1:
        result = result[0].get(columnNames[0])
    return result
def get_download_limit(search_map):
    download_limit = getAnyCombo(search_map,"download_limit")
    return int(download_limit or 0)
def get_download_count(search_map):
    download_count = getAnyCombo(search_map,"download_count")
    return int(download_count or 0)
def is_download_limit(search_map):
    download_limit = get_download_limit(search_map)
    if download_limit == 0:
        return False
    return download_limit
def is_download_limit_exceeded(search_map):
    download_limit = is_download_limit(search_map)
    if download_limit == False:
        return False
    download_count = get_download_count(search_map)
    if download_limit > download_count:
        return False
    return True
def add_to_download_count(search_map):
    download_count = get_download_count(search_map)
    update_any_combo(table_name='uploads',
                                  update_map={"download_count":download_count+1},
                                  search_map=search_map)

def check_password(pwd_given: Optional[str], share_password: Optional[str]) -> Tuple[bool, Optional[str]]:
    if not share_password:
        return True, None
    if not pwd_given:
        return False, 'PASSWORD_REQUIRED'
    if not verify_password(pwd_given, share_password):
        return False, 'PASSWORD_INCORRECT'
    return True, None
def getRowValue(req=None,data=None,search_map=None,row=None,key=None):
   search_map,data,row = get_searchmap_data_row(req=req,data=data,search_map=search_map,row=row)
   logger.info(f"row == {row}\n{key} == {row.get(key)}")
   if key:
       return row.get(key)
def update_if_not_None(data={},update_data={}):
   for key,value in update_data.items():
      initValue = data.get(key)
      if value != None and initValue == None:
        data[key] = value
   return data
def get_all_args(req):
    if req is None:
        return {}

    # If we already got a dict, just return it
    if isinstance(req, dict):
        return dict(req)

    # Otherwise assume it's a Flask request
    all_args = {}
    result = extract_request_data(req) or {}
    all_args.update(result.get("json", {}) or {})
    all_args.update(req.view_args or {})
    all_args.update(req.args or {})
    all_args.update(req.form or {})
    all_args.update(req.get_json(silent=True) or {})
    if req.files:
        for key, storage in req.files.items():
            all_args[key] = storage
    return all_args

def get_search_map_data(req=None,data=None,search_map=None):
   search_map = search_map or {}
   data = data or {}
   search_map_vars = ['id',"uploader_id","filepath"]
   all_args = get_all_args(req)
   data = update_if_not_None(data,all_args)
   data = update_if_not_None(data,search_map)
   user_id, user_err = get_user_id_from_request(request)
   data = update_if_not_None(data,{'user':user_id})
   for key in search_map_vars:
      dataVar = data.get(key)
      search_map = update_if_not_None(search_map,{key:dataVar})
   return search_map,data
def get_searchmap_data_row(req=None,data=None,search_map=None,row=None):
    search_map,data = get_search_map_data(req=req,data=data,search_map=search_map)
    logger.info(f"SEARCHMAP==={search_map}")
    if row == None:
        rows = fetch_any_combo(column_names='*', table_name='uploads', search_map=search_map)
        rows = rows or {}
        if isinstance(rows, list) and len(rows)>0:
            for row in rows:
                logger.info(f"ROW === {row}")
                allVars = True
                for key,value in search_map.items():
                    if str(row.get(key,'')) != str(value):
                        allVars = False
                        break
                if allVars:
                    break        
    data = update_if_not_None(data,{'filepath':row.get('filepath')})
    return search_map,data,row
def get_download(req=None,data=None,search_map=None,row=None,username=None):
    search_map,data,row = get_searchmap_data_row(req=req,data=data,row=row,search_map=search_map)

    filepath = getRowValue(
        req=req,
        data=data,
        search_map=search_map,
        row=row,
        key='filepath'
        )
    uploader_id = getAnyCombo(search_map,['uploader_id'])

    logger.info(f"search_map == {search_map}")
    logger.info(f"data == {data}")
    logger.info(f"row == {row}")
    logger.info(f"uploader_id == {uploader_id}")
    abs_path = get_full_path(filepath,user=uploader_id,row=row,req=req,data=data)
    abs_dir = os.path.dirname(abs_path)
    basename = os.path.basename(filepath)
    filename,ext = os.path.splitext(basename)

    if not abs_path:
        return abs_path, filename, 'NO_FILE_FOUND'
    is_user = is_user_uploader(req=req, data=data)
    if is_user:
        return abs_path, filename, None
    shareable = getRowValue(
        req=req,
        data=data,
        search_map=search_map,
        row=row,
        key='shareable'
        )
    if not shareable:
        return abs_path, filename, 'NOT_SHAREABLE'
    if is_download_limit_exceeded(search_map) == True:
        return abs_path, filename, 'DOWNLOAD_LIMIT'

    share_password = getRowValue(req=req, data=data, search_map=search_map, row=row, key='share_password')

    # IMPORTANT: pass req so form pwd is seen
    init = get_file_inits(req=req, data=data)      # ← was get_file_inits(data=data)
    ok, err = check_password(init.get('pwd'), share_password)
    if not ok:
        return abs_path, filename, err
    return abs_path, filename, None

def get_row(search_map, column_names="*", table_name="uploads",username=None,req=None):
    """
    Fetches a single row from the database based on search criteria.

    Args:
        data (dict): Request data to build the search map.
        column_names (str, optional): Columns to select. Defaults to "*".
        table_name (str, optional): Table to query. Defaults to "uploads".

    Returns:
        tuple: (row_data, status_code)
            - row_data (dict or str): Fetched row or error message.
            - status_code (int): HTTP status code.
    """
    

    

    rows = fetch_any_combo(
        column_names=column_names,
        table_name=table_name,
        search_map=search_map
    )

    if not rows:
        msg = {}
        msg["error"] = "No matching row found"
        msg["status_code"] = 404
        msg["success"] = False
        return msg,400
    if rows and isinstance(rows,list) and len(rows) == 1:
        rows = rows[0]
    return rows, 200

def secure_remove(data, username=None, column_names="*", table_name="uploads",req=None):
    """
    Removes a file record from the database if the user is authorized.

    Args:
        data (dict): Request data containing file ID or path.
        username (str): Username of the requester.
        column_names (str, optional): Columns to select. Defaults to "*".
        table_name (str, optional): Table to query. Defaults to "uploads".

    Returns:
        tuple: (message, status_code)
            - message (str or bool): Result message or True if successful.
            - status_code (int): HTTP status code.
    """
    search_map = {}
    for key in ["id","filename","uploader_id","filepath"]:
        value = data.get(key)
        if value:
            search_map[key] = value
    if not search_map:
        return get_output(error=msg.get("error"),status_code=400,sent=True) 
    row, status_code = get_row(search_map, column_names=column_names, table_name=table_name,username=username,req=req)
    if status_code != 200:
        return row, status_code

    uploader_id = row.get("uploader_id")
    logger.info(f"uploader_id==={uploader_id}")
    logger.info(f"username==={username}")
    if uploader_id.lower() != username.lower():
        return f"Unauthorized user: {uploader_id}", 403

    
    try:
        remove_any_combo(table_name=table_name, search_map=search_map)
        return True, 200
    except Exception as e:
        logger.error(f"Failed to remove file: {e}")
        return f"Removal failed: {str(e)}", 500



from .files import (
    secure_env_bp,
    secure_files_bp,
    secure_upload_bp,
    secure_remove_bp,
    secure_register_bp,
    secure_download_bp
    )
from .settings import (
    secure_settings_bp,
    secure_endpoints_bp
    )
from .users import (
    secure_logout_bp,
    secure_login_bp,
    change_passwords_bp,
    secure_users_bp
    )
from .views import secure_views_bp
